import { UpsertDialogInputConfig } from "../UpsertDialog/types";
import useUpsertDialog from "../UpsertDialog/useUpsertDialog";
import MyTable from "./MyTable";
import { MyTableConfig } from "./MyTable/types";
import { MaintenanceEntry } from "./types";
import useData from "./useData";

const tableConfig: MyTableConfig<MaintenanceEntry> = [
  {
    heading: "Parts",
    keyInData: "parts",
  },
  {
    heading: "Mileage",
    keyInData: "mileage",
  },
  {
    heading: "Date",
    keyInData: "date",
  },
  {
    heading: "Cost",
    keyInData: "cost",
  },
];

export default function CarMaintenanceHistory() {
  const { data, handleInsertion, handleEdit } = useData<MaintenanceEntry>();

  const inputConfig: UpsertDialogInputConfig<MaintenanceEntry> = [
    {
      name: "parts",
      placeholder: "Parts",
      options: {
        required: "Field required",
      },
    },
    {
      name: "mileage",
      placeholder: "Mileage",
      options: {
        required: "Field required",
        validate: (v) => !!Number.parseInt(v) || 'Mileage must be a number',
      },
    },
    {
      name: "date",
      placeholder: "Date",
      options: {
        validate: (v) => {
          if (v && new Date(v).toString() === "Invalid Date") {
            return "Invalid date format";
          }
          return true;
        },
      },
    },
    {
      name: "cost",
      placeholder: "Cost",
      options: {
        validate: (v) => {
          if (v && !v.match(/^\d+([\\.]\d+)*$/g)) {
            return "Cost has to be a number";
          }
          return true;
        },
      },
    },
  ];

  const handleGetInitialData: (id: string | number | boolean) => Promise<MaintenanceEntry | undefined> =
    async (id) => {
      return new Promise((resolve) => {
        const item = data.find((item) => item.id === id);
        if (item && item.date) {
          item.date = new Date(item.date).toLocaleDateString('en');
        }
        setTimeout(() => resolve(item), 2000);
      });
    }

  const handleOnSubmit: (id: string | number | boolean, data: MaintenanceEntry) => Promise<void> =
    async (id, data) => {
      const isEditing = typeof id !== "boolean";
      if (data.date) {
        data.date = new Date(data.date).toDateString();
      }
      isEditing
        ? handleEdit(id as number, data)
        : handleInsertion(data);
      return Promise.resolve();
    }

  const { UpsertDialog, openDialog } = useUpsertDialog<MaintenanceEntry>({
    title: "Car Maintenance",
    inputConfig,
    getInitialData: handleGetInitialData,
    onSubmit: handleOnSubmit,
  });

  return (
    <>
      <MyTable
        config={tableConfig}
        data={data}
        onNewItem={() => openDialog()}
        onEditItem={(id) => openDialog(id)}
      />
      <UpsertDialog />
    </>
  );
}
