import { UpsertDialogInputConfig } from "../UpsertDialog/types";
import useUpsertDialog from "../UpsertDialog/useUpsertDialog";
import MyTable from "./MyTable";
import { MyTableConfig } from "./MyTable/types";
import { ChemicalElementEntry } from "./types";
import useData from "./useData";

const tableConfig: MyTableConfig<ChemicalElementEntry> = [
  {
    heading: "Name",
    keyInData: "name",
  },
  {
    heading: "Symbol",
    keyInData: "symbol",
  },
  {
    heading: "Atomic Number",
    keyInData: "atomicNumber",
  },
  {
    heading: "Atomic Mass",
    keyInData: "atomicMass",
  },
  {
    heading: "Is Synthetic",
    keyInData: "synthetic",
  },
];

const inputConfig: UpsertDialogInputConfig<ChemicalElementEntry> = [
  {
    name: "name",
    placeholder: "Name",
    options: {
      required: "Field required",
    },
  },
  {
    name: "symbol",
    placeholder: "Symbol",
    options: {
      required: "Field required",
      maxLength: {
        value: 2,
        message: 'Symbol must be maximum 2 characters long'
      },
      pattern: {
        value: /^[a-zA-Z]+$/,
        message: 'Symbol must contain only letters',
      }
    },
  },
  {
    name: "atomicNumber",
    placeholder: "Atomic Number",
    options: {
      required: "Field required",
      max: {
        value: 118,
        message: 'Maximum number allowed is 118'
      },
      min: {
        value: 1,
        message: 'Minimum number allowed is 1'
      },
      pattern: {
        value: /^\d+$/g,
        message: 'Atomic number must be a positive integer number',
      }
    },
  },
  {
    name: "atomicMass",
    placeholder: "Atomic Mass",
    options: {
      validate: (v) => {
        if (v && !v.match(/^\d+([\\.]\d+)*$/g)) {
          return 'Atomic number must be a valid number';
        }
        return true;
      },
    },
  },
  {
    type: "checkbox",
    placeholder: "Artificially produced",
    name: "synthetic",
  }
];

export default function ChemicalElementsLearnt() {
  const { data, handleInsertion, handleEdit } = useData<ChemicalElementEntry>();

  const handleGetInitialData: (id: string | number | boolean) => Promise<ChemicalElementEntry | undefined> =
    async (id) => {
      return new Promise((resolve) => {
        const item = data.find((item) => item.id === id);
        setTimeout(() => resolve(item), 2000);
      });
    }

  const handleOnSubmit: (id: string | number | boolean, data: ChemicalElementEntry) => Promise<void> =
    async (id, data) => {
      data.synthetic = !!data.synthetic;
      const isEditing = typeof id !== "boolean";
      isEditing
        ? handleEdit(id as number, data)
        : handleInsertion(data);
      return Promise.resolve();
    }

  const { UpsertDialog, openDialog } = useUpsertDialog<ChemicalElementEntry>({
    title: "Chemical Element",
    inputConfig,
    getInitialData: handleGetInitialData,
    onSubmit: handleOnSubmit,
  });

  return (
    <>
      <MyTable
        config={tableConfig}
        data={data}
        onNewItem={() => openDialog()}
        onEditItem={(id) => openDialog(id)}
      />
      <UpsertDialog />
    </>
  );
}
