import { useEffect } from "react";
import {
    Box,
    Button,
    CircularProgress,
    Dialog,
    DialogActions,
    DialogContent,
    DialogTitle,
    Typography,
} from "@mui/material";
import { UpsertDialogProps } from "./types";
import { useForm } from "react-hook-form";
import UpsertDialogInput from "./UpsertDialogInput";

const TRANSITION_DELAY = 300

export const LoadingDialog = () => (
    <Dialog open={true} transitionDuration={{ exit: TRANSITION_DELAY }}>
        <CircularProgress sx={{ margin: 4 }} />
    </Dialog>
);

function UpsertDialog<T>({
    inputConfig,
    title,
    isOpen,
    onSubmit,
    onClose
}: UpsertDialogProps<T>) {
    const {
        register,
        control,
        handleSubmit,
        formState: { errors },
        setValue,
        reset,
    } = useForm();

    const _onSubmit = (data: any) => {
        // (Optional!) Cleanup of empty fields
        for (const key in data) {
            if (!data[key]) {
                delete data[key];
            }
        }
        _onClose();
        onSubmit(data);
    };

    const isFormDisabled = errors && Object.keys(errors).length > 0;
    const isEditing = inputConfig != null && inputConfig.some((item) => item.initialValue);

    useEffect(() => {
        if (isEditing) {
            for (const item of inputConfig) {
                if (item.initialValue) {
                    setValue(item.name, item.initialValue);
                }
            }
        }
    }, [inputConfig, isEditing, setValue]);


    const _onClose = () => {
        onClose();
        reset();
    };

    return inputConfig && (
        <Dialog
            open={isOpen}
            transitionDuration={{ appear: TRANSITION_DELAY }}
            fullWidth
            onClose={(_, reason) => {
                if (reason !== 'backdropClick') { // Optional!
                    _onClose();
                }
            }}
        >
            {title && <DialogTitle>{title}</DialogTitle>}
            <form onSubmit={handleSubmit(_onSubmit)}>
                <DialogContent>
                    {inputConfig.map((item) => (
                        <Box my={1} key={item.name}>
                            <UpsertDialogInput
                                type={item.type}
                                configItem={item}
                                hasErrors={!!errors[item.name]}
                                register={register}
                                control={control}
                            />
                            {!!errors[item.name] && (
                                <Typography color="error" fontSize={12}>
                                    {errors[item.name]?.message?.toString()}
                                </Typography>
                            )}
                        </Box>
                    ))}
                </DialogContent>
                <DialogActions sx={{ mx: 2, mb: 2 }}>
                    <Button variant="text" color="error" onClick={_onClose} disabled={isFormDisabled}>
                        Cancel
                    </Button>
                    <Button variant="contained" type="submit">
                        Submit
                    </Button>
                </DialogActions>
            </form>
        </Dialog>
    );
}

export default UpsertDialog;
