import { useEffect, useReducer, useRef } from "react";
import UpsertDialog, { LoadingDialog } from ".";
import upsertDialogReducer, {
    CLOSE_DIALOG,
    OPEN_DIALOG,
    initialState as upsertDialogInitialState,
} from "./upsertDialogReducer";
import { UseUpsertDialog } from "./types";

export function useUpsertDialog<T extends object>({
    title,
    inputConfig,
    getInitialData,
    onSubmit,
}: UseUpsertDialog<T>) {
    const [dialogState, dispatchAction] = useReducer(
        upsertDialogReducer,
        upsertDialogInitialState
    );
    const dataFetchRequested = useRef(false);

    useEffect(() => {
        async function getInitialValues() {
            const editData = await getInitialData(dialogState.id);
            if (!editData) return;
            const config = [...inputConfig];
            for (const field in editData as T) {
                const configItemIndex = config.findIndex((item) => item.name === field);
                let configItem = config[configItemIndex];
                if (!configItem) continue;
                configItem = { ...config[configItemIndex] };
                configItem.initialValue = editData[field];
                config.splice(configItemIndex, 1, configItem);
            }
            dispatchAction({
                type: OPEN_DIALOG,
                payload: {
                    config,
                    id: dialogState.id,
                },
            });
        }

        if (typeof dialogState.id !== "boolean" && !dataFetchRequested.current) {
            getInitialValues();
            dataFetchRequested.current = true;
        }
    }, [dialogState.id, getInitialData, inputConfig]);

    const handleCloseDialog = () => {
        dispatchAction({ type: CLOSE_DIALOG });
        dataFetchRequested.current = false;
    };

    const handleOpenDialog = (id?: string | number | boolean) => {
        dispatchAction({
            type: OPEN_DIALOG,
            payload: {
                config: id ? null : [...inputConfig],
                id: id || true,
            },
        });
    };

    const _onSubmit = async (data: T) => {
        await onSubmit(dialogState.id, data);
    };

    const isLoading = !!dialogState.id && !dialogState.config;

    return {
        UpsertDialog: () =>
            isLoading ? (
                <LoadingDialog />
            ) : (
                <UpsertDialog
                    title={title}
                    inputConfig={dialogState.config}
                    isOpen={!!dialogState.id}
                    onClose={handleCloseDialog}
                    onSubmit={_onSubmit}
                />
            ),
        openDialog: handleOpenDialog,
    };
}

export default useUpsertDialog;
